/**Common definitions and configuration settings for the Alarm application
 *
 * 20151221		0.1		Creation, refactored from main.c
 */

#ifndef __ALARM_H_
#define __ALARM_H_

#include "FreeRTOS.h"
#include "task.h"
#include "utils.h"
#include "gpio.h"
#include "gpioint.h"
#include "fsm.h"


/* system-wide configuration settings */
enum
{
	cfg_logPort				= UART_port0, 	/* disable logging with UART_portNone, */
	cfg_logPortBauds		= 56000			/* logger communication speed in bauds */
};

/**Task in charge of handling user input
 * This is a FreeRTOS task, hence the function signature.
 * Depends on the the KEYPAD and CMD modules.
 *
 * @param	pvParameters	internally casted to AlarmState*, allowing access to required alarm state.
 *
 */
void taskCmd(void* pvParameters);

/**Task in charge of maintaining the FSM.
 * This is a FreeRTOS task, hence the function signature.
 * Depends on the the FSM, FSMSTATES, and LEDCTRL modules.
 *
 * @param	pvParameters	internally casted to AlarmState*, allowing access to required alarm state.
 *
 */
void taskFsm(void* pvParameters);

#endif /* __ALARM_H_ */
