/**Entry point & top level code for the Alarm application
 *
 * 20151223		0.9		Cleanup, configuration edits
 * 20151220		0.2		Queue, tasks added
 * 20151213		0.1		First implementation
 */


#include "FreeRTOS.h"
#include "task.h"
#include "log.h"
#include "printf.h"
#include "ledctrl.h"
#include "fsm.h"
#include "sensors.h"
#include "siscom.h"
#include "alarm.h"
#include <assert.h>
#include <string.h>


int main(void)
{
	static AlarmState alarmState = {{ 0 }};
	Result result;

	/* initialize logger */
	if(cfg_logPort != (uint32_t)UART_portNone)
	{
		result = PRINTF_init(cfg_logPort);
		assert(result == Result_OK);
		result = UART_init(cfg_logPort, cfg_logPortBauds, UART_modeWithInterrupts);
		assert(result == Result_OK);
		result = LOG_init(cfg_logPort);
		assert(result == Result_OK);
	}
	LOG_log("logger initialized");

	/* initialize siscom module */
	SISCOM_init();
	LOG_log("siscom module initialized");

	/* alarm state initialization, step 1. alarmState->fsmAlarm is initialized @ taskCmd() */
	strcpy(alarmState.password, "1234");

	alarmState.queueHandle = xQueueCreate(10, sizeof(uint32_t));
	assert(alarmState.queueHandle != NULL);
	LOG_log("alarm state initialized");

	/* initialize sensors */
	GPIOINT_init();
//	SENSORS_init(&alarmState.fsmAlarm.sensorStateMagneticContact, &alarmState.fsmAlarm.sensorStatePassiveInfrared, &alarmState);
	SENSORS_init(&alarmState);
	FSM_logSensorStates(&alarmState.fsmAlarm);
	LOG_log("sensors initialized");

	/* initialize ledctrl module */
	result = LEDCTRL_init(100, GPIO_Port0, 22, GPIO_Port0, 21);		/* upon initialization, this module creates a task */
	LEDCTRL_config(LEDCTRL_LedRed,   "01010101", 0, 10);
	LEDCTRL_config(LEDCTRL_LedGreen, "10101010", 0, 10);
	LOG_log("ledctrl module initialized");

	/* initialized tasks */
	BaseType_t taskCreationResult = xTaskCreate(taskCmd, "cmd", 2000, &alarmState, tskIDLE_PRIORITY+2, NULL);
	assert(taskCreationResult == pdTRUE);
	taskCreationResult = xTaskCreate(taskFsm, "fsm", 2000, &alarmState, tskIDLE_PRIORITY+2, NULL);
	assert(taskCreationResult == pdTRUE);
	LOG_log("tasks initialized");

	/* start scheduler */
	LOG_log("starting scheduler...");
	vTaskStartScheduler();
	return 0; /* unreachable */
}
