/**FSMSTATE module. Module specific to the Alarm application.
 * Implementation for each state of the alarm application.
 *
 * Depends on FSM, SISCOM and LEDCTRL.
 *
 * 20151227		1.0		Usage of with siscom
 * 20151223		0.9		Integration with alarm
 * 20151217		0.8		Refactor into its own files
 * 20151214		0.2		State refinemnents
 * 20151208		0.1		First states
 */


#ifndef __FSMSTATE_H
#define __FSMSTATE_H


#include "fsm.h"

/**This applies to all functions below:
 *
 * They implement the behavior for a given state, which involves:
 * - Handling events as they are received. Causing FSM to transition from the current state to another.
 * - Executing actions to be carried during the transitions into a another state.
 * - If no transitions are needed (most common case), executing the required housekeeping actions, if any.
 *
 * @param	as		the alarm state
 * @param	event	the newly received event
 * 					or, if no event has been received, Event_None
 */

/*See above comment. This handles the disarmed state */
void FSMSTATE_disarmed(FSM_Alarm* as, FSM_Event event);

/*See above comment. This handles the prearmed state */
void FSMSTATE_preArmed(FSM_Alarm* as, FSM_Event event);

/*See above comment. This handles the armed state */
void FSMSTATE_armed(FSM_Alarm* as, FSM_Event event);

/*See above comment. This handles the prealarm state */
void FSMSTATE_preAlarm(FSM_Alarm* as, FSM_Event event);

/*See above comment. This handles the alarm state */
void FMSSTATE_alarm(FSM_Alarm* as, FSM_Event event);

#endif /* __FSMSTATE_H */
