/**SISCOM module. Module specific to the Alarm application.
 * Allows interfacing with a bidirection GSM/GPRS communitation system (aka siscom).
 * Handles initialization and mapping of discrete GPIO pins to siscom inputs.
 *
 * This module depends on the GPIO modules.
 *
 * 20151228		0.9		Bug fixes, integration
 * 20151222		0.1		First implementation
 */

#ifndef __SISCOM_H
#define __SISCOM_H


/**Different messages, mapped to the siscom inputs */
typedef enum
{
	/* siscom imput connections */
	SiscomMessage_AlarmMC,
	SiscomMessage_AlarmPIR,
	SiscomMessage_DisconnectedUnderCoaction,
	SiscomMessage_Arm,
	SiscomMessage_Disarm,
	/* helper */
	SiscomMessage_Num
}	SISCOM_Message;


/**Initializes the module
 * Just sets all siscom inputs to their default values
 *
 * @return							see Result
 */
Result SISCOM_init(void);

/**Activates the siscom module input
 * Sends a short pulse to the
 *
 * @param		siscomMessage		message Id that will be mapped to the appropriate siscom input
 *
 * @return							see Result
 */
Result SISCOM_notify(SISCOM_Message siscomMessage);

#endif /* __SISCOM_H */
