#include "FreeRTOS.h"
#include "task.h"
#include "log.h"
#include "gpio.h"
#include "gpioint.h"
#include "utils.h"
#include "sensors.h"
#include <assert.h>
#include <stdio.h>
#include <string.h>


void SENSORS_intrHandlerMC(void* params, uint8_t pinValue)
{
	if(params)
	{
		AlarmState* alarmState = (AlarmState*) params;
		if(alarmState->queueHandle)
		{
			FSM_Event event = pinValue ? Event_Sensor_MagneticContact_On : Event_Sensor_MagneticContact_Off;
			xQueueSendToBackFromISR (alarmState->queueHandle, &event, NULL);
		}
	}
}

void SENSORS_intrHandlerPIR(void* params, uint8_t pinValue)
{
	if(params)
	{
		AlarmState* alarmState = (AlarmState*) params;
		if(alarmState->queueHandle)
		{
			FSM_Event event = pinValue ? Event_Sensor_PassiveInfrared_On : Event_Sensor_PassiveInfrared_Off;
			xQueueSendToBackFromISR (alarmState->queueHandle, &event, NULL);
		}
	}
}

Result SENSORS_configSensor(GPIOINT_Slot slot, GPIO_Port port , uint32_t pin, void (*intrHandler)(void*, uint8_t), void* params, FSM_SensorState* sensorState, uint8_t enabled, const char* description)
{
	if(!GPIOINT_isInitialized())
	{
		return Result_Failed_NotInitialized;
	}

	Result result = GPIO_configPinAsInput(port, pin, PinMode_PullUp);
	if(result == Result_OK)
	{
		GPIOINT_cfg cfg = { intrHandler, params, port, pin, GPIOINT_EdgeKind_RaisingORFalling };
		result = GPIOINT_set(slot, &cfg);

		strcpy(sensorState->description, description);
		sensorState->detecting = (0 != GPIO_getPinValue(port, pin));
		sensorState->enabled = enabled;
	}
	return result;
}

Result SENSORS_init(AlarmState* alarmState)
{
	if(!GPIOINT_isInitialized())
	{
		return Result_Failed_NotInitialized;
	}

	FSM_SensorState* mc = &alarmState->fsmAlarm.sensorStateMagneticContact;
	FSM_SensorState* pir = &alarmState->fsmAlarm.sensorStatePassiveInfrared;
	Result result = SENSORS_configSensor(GPIOINT_Slot0, GPIO_Port2, 13, SENSORS_intrHandlerMC, alarmState, mc, 0, "MC ");
	if( result == Result_OK)
	{
		result = SENSORS_configSensor(GPIOINT_Slot1, GPIO_Port0, 26, SENSORS_intrHandlerPIR, alarmState, pir, 0, "PIR");
	}
	return result;
}

Result SENSORS_deinit(void)
{
	if(!GPIOINT_isInitialized())
	{
		return Result_Failed_NotInitialized;
	}

	Result result = GPIOINT_unset(GPIOINT_Slot0);
	if(result == Result_OK)
	{
		result = GPIOINT_unset(GPIOINT_Slot1);
	}
	return result;
}
