#include "FreeRTOS.h"
#include "task.h"
#include "log.h"
#include "gpio.h"
#include "utils.h"
#include "siscom.h"
#include <assert.h>


uint8_t SISCOM_initialized = 0;


uint8_t SISCOM_isInitialized(void)
{
	return SISCOM_initialized != 0;
}


char* SISCOM_GetPattern(SISCOM_Message siscomMessage)
{
	assert(siscomMessage < SiscomMessage_Num);
	static char* r[SiscomMessage_Num] = { "10", "10", "10", "1", "0" };
	return r[siscomMessage];
}

GPIO_Port SISCOM_GetSiscomPort(SISCOM_Message siscomMessage)
{
	assert(siscomMessage < SiscomMessage_Num);
	GPIO_Port r[SiscomMessage_Num] = { GPIO_Port0, GPIO_Port0, GPIO_Port1, GPIO_Port1, GPIO_Port1 };
	return r[siscomMessage];
}

uint32_t SISCOM_GetSiscomPin(SISCOM_Message siscomMessage)
{
	assert(siscomMessage < SiscomMessage_Num);
	uint32_t r[SiscomMessage_Num] = { 24, 25, 30, 31, 31 };
	return r[siscomMessage];
}

Result SISCOM_notify(SISCOM_Message siscomMessage)
{
	if(!SISCOM_isInitialized())
	{
		return Result_Failed_AlreadyInitialized;
	}
	if(siscomMessage >= SiscomMessage_Num)
	{
		return Result_Failed_InvalidParams;
	}

	char* description[SiscomMessage_Num] =
	{
		"Burglary alarm (MC)",
		"Burglary alarm (PIR)",
		"Disconnection under coaction",
		"Alarm armed",
		"Alarm disarmed"
	};
	char tmp[30];
	LOG_log
	(
		"%sSending SMS message #%d from SISCOM: %s%s",
		makeAnsiColors(Color_BrightMagenta, Color_Black, tmp),
		(uint8_t) siscomMessage,
		description[siscomMessage],
		makeAnsiResetAttributes()
	);
	
	char* pattern = SISCOM_GetPattern(siscomMessage);
	int i;
	for(i=0 ; pattern[i] ; i++)
	{
		GPIO_setPinValue(SISCOM_GetSiscomPort(siscomMessage), SISCOM_GetSiscomPin(siscomMessage), pattern[i] != '0');
		vTaskDelay(msToTicks(10)); 		/* TODO: this will block current task! */
	}
	return Result_OK;
}

Result SISCOM_init(void)
{
	if(SISCOM_isInitialized())
	{
		return Result_Failed_AlreadyInitialized;
	}
	SISCOM_initialized = 1;

	int i;
	for(i=0 ; i<SiscomMessage_Num ; i++)
	{
		GPIO_configPinAsOutput(SISCOM_GetSiscomPort(i), SISCOM_GetSiscomPin(i));
		GPIO_setPinValue(SISCOM_GetSiscomPort(i), SISCOM_GetSiscomPin(i), 0);
	}
	return Result_OK;
}
