/**ARPALAB module
 * Sends requests to Arp@lab REST API through the WIFLY module
 * Depends on PARSING, WIFLY, UART, PRINTF, LOG modules
 *
 * 20151110		1.0		Many fixes
 * 20151108		0.9		Creation
 */

#ifndef __ARPALAB_H
#define __ARPALAB_H

#include "utils.h"


/**Initializes the module
 * There is no web interaction in the initialization
 *
 * @param	nid		Network ID. Arbitrary value
 * @param	ip		IP address. Has to match emitter's IP
 * @param	app		String to name the application
 *
 * @return	Result_OK							On success
 * 			Result_Failed_AlreadyInitialized	If module is already initialized
 * 			Result_Failed_InvalidParams			If unusable parameters where provided
 * */
Result ARPALAB_init(char* nid, char* ip, char* app);

/**Deinitializes the module
 * There is no web interaction in the initialization
 *
 * @return	Result_OK							On success
 * 			Result_Failed_NotInitialized		If module was not initialized
 * */
Result ARPALAB_deinit();

/**Gets a random value from arpalab
 *
 * @param	value		receiver for the value, if the operation succeeds
 *
 * @return	Result_OK							On success
 * 			Result_Failed			  			On Failure
 * 			Result_Failed_NotInitialized		If module was not initialized
 * */
Result ARPALAB_getRandomValue(uint32_t* value);

/**Sends a given value to arpalab
 *
 * @param	value		the value to send
 *
 * @return	Result_OK							On success
 * 			Result_Failed			  			On Failure
 * 			Result_Failed_NotInitialized		If module was not initialized
 * */
Result ARPALAB_sendValue(uint32_t value);


#endif /* __ARPALAB_H */
