/**CIRCULARBUFFER module.
 *
 * This is not a singleton module. Many instances of CIRCULARBUFFER_data are allowed.
 *
 * 20151227		0.9		Documentation, edits
 * 20151220		0.8		Many fixes
 * 20151218		0.7		Creation
 */

#ifndef __CIRCULARBUFFER_H
#define __CIRCULARBUFFER_H

#include "utils.h"

/* Circular buffer data */
typedef struct
{
    char* buffer;     /* data buffer */
    char *end;        /* end of data buffer */
    char* head;       /* pointer to head */
    char* tail;       /* pointer to tail */
    size_t size;      /* maximum number of items in the buffer */
    size_t count;     /* number of items in the buffer */
}	CIRCULARBUFFER_data;


/**Initializes an instance of CIRCULARBUFFER_data (not the CIRCULARBUFFER module itself).
 *
 * @param	cb		the data structure that holds the complete state of a circular buffer
 * @param	buffer	user-allocated buffer that will contain the actual data.
 *					the buffer has to be allocated by the user, and his scope must be at least the CIRCULARBUFFER_data's.
 * @param	size	size of buffer
 */
void CIRCULARBUFFER_init(CIRCULARBUFFER_data* cb, char* buffer, size_t size);

/**Pushes back one item into the circular buffer
 * @param	cb		the circular buffer
 * @param	item	data item to insert
 *
 * @return			see Result
 */
Result CIRCULARBUFFER_pushBack(CIRCULARBUFFER_data* cb, char item);

/**Front-pops one item from the circular buffer
 * @param	cb		the circular buffer
 * @param	item	data item to read
 *
 * @return			see Result
 * 					if buffer is empty, return is Result_Failed
 */
Result CIRCULARBUFFER_popFront(CIRCULARBUFFER_data* cb, char* item);

/**Resets a circular buffer
 *
 * @param	cb		the circular buffer
 *
 * @return			see Result
 */
Result CIRCULARBUFFER_clean(CIRCULARBUFFER_data* cb);

/**Converts the circular buffer to a ASCIIZ string
 *
 * @param	cb		the circular buffer
 * @param	buffer	pointer to a string that will receive the string
 * 					this should point to a buffer large enough to accomodate the biggest possible
 * 					outcome, say the initial size set @ set CIRCULARBUFFER_init
 */
void CIRCULARBUFFER_toString(CIRCULARBUFFER_data* cb, char* buffer);

#endif /* __CIRCULARBUFFER_H */
