/* GPIO module
 * Support for general purpose input/output
 *
 *	20151122	0.80	 Initial version
 */

#ifndef __GPIO_H
#define __GPIO_H

#include "utils.h"

/* input pin configuration */
typedef enum
{
	PinMode_PullUp,					/* use built-in pull up resistor */
	PinMode_Repeater,				/* use repeater mode */
	PinMode_NoPullUpNoPullDown,		/* use neither built-in pull up nor pull-down resistors */
	PinMode_PullDown				/* use built-in pull down resistor */
}	GPIO_PinMode;

/* all available ports */
typedef enum
{
	GPIO_Port0,
	GPIO_Port1,
	GPIO_Port2,
	GPIO_Port3,
	GPIO_Port4
}	GPIO_Port;


/**Get a pin value
 *
 * @param	port	port to read from
 * @param	pin		pin to read from
 *
 * return			either 0 or 1
 */
uint32_t GPIO_getPinValue(GPIO_Port port, uint8_t pin);

/**Set a pin value
 *
 * @param	port	port to write to
 * @param	pin		pin to write to
 * @param	value	value to write
 */
void GPIO_setPinValue(GPIO_Port port, uint8_t pin, uint8_t value);

/**Configure a single pin a output
 *
 * @param	port	port to read from
 * @param	pin		pin to read from
 *
 * return			see Result
 */
Result GPIO_configPinAsOutput(GPIO_Port port, uint8_t pin);

/**Configure a single pin a input
 *
 * @param	port		port to write to
 * @param	pin			pin to write to
 * @param	pinMode		desired input pin operation mode. See GPIO_PinMode
 *
 * return				see Result
 */
Result GPIO_configPinAsInput(GPIO_Port port, uint8_t pin, GPIO_PinMode pinMode);

/**Configure a pin as open drain
 *
 * @param	port			port to set
 * @param	pin				pin to set
 * @param	openDrainMode	desired pin operation mode
 * 							it can be either 0 or 1. 0 means "normal" (no open drain), 1 means "open drain"
 *
 * return					see Result
 */
Result GPIO_SetOpenDrainMode(GPIO_Port port, uint8_t pin, uint8_t openDrainMode);

#endif /* __GPIO_H */
