/**
 * PARSING module
 * Simple parsing facilities
 * Parsing is destructive. Expect any non-const parameters to be modified or thrashed
 * Rationale for this implementation is the memory usage constrains
 *
 * 20151101		1.0		PARSING_removeTags added
 * 20151024		0.8		PARSING_splitLines added
 * 20151021		0.1		creation
 */


#ifndef __PARSING_H_
#define __PARSING_H_

#include "utils.h"


/**Checks whether a match is in the exact location in a given string
 *
 * @param	buffer	String where to search for the match
 * @param	start	Position where the match is expected
 * 					Can be negative. Meaning that the starting position is relative to the end of string
 * @param	match	String to look for in buffer
 *
 * @return	Result_OK						If match is found
 * 			Result_Failed_InvalidParams		If parameters are unusable
 * 			Result_Failed					If match is not found, or unexpected error
 * */
Result PARSING_checkSubst(const char* buffer, int start, const char* match);

/**Splits a string with arbitrarily CR or LF characters into lines
 * Replaces Any CR of LF character by \0. Note that this is destructive parsing
 * Computes a set of lines within the buffer
 *
 * @param	buffer	String to split
 * @param	lines	Array of pointers where to return a pointer to the start of each line
 * 					Can be negative. Meaning that the starting position is relative to the end of string
 * @param	linesSize	Maximum number of lines to return
 * 					It is ok if there are less than the actual lines in buffer
 *
 * @return	Result_OK						If operation was completed with no errors
 * 			Result_Failed_InvalidParams		If parameters are unusable
 * 			Result_Failed_NotEnoughMemory	If there is no room in lines to accomodate all lines
 *
 * It is possible to call this function with both lines==NULL and lineSize==0, just to get the
 * feature of zeroing CR / LF
 * */
Result PARSING_splitLines(char* buffer, char** lines, size_t linesSize);

/**Removes left and/or right tags (substrings) from a given string
 *
 * @param	buffer	String to have tags removed
 * @param	left	Tag at the left of the substring we're interested in
 * @param	right	Tag at the right of the substring we're interested in
 *
 * @return	A pointer to the found substring, or NULL if not found
 *
 * It is ok for left and right to be empty, but they can't be null
 */
char* PARSING_removeEnclosingTags(char* buffer, const char* left, const char* right);


#endif /* __PARSING_H_ */
