/*	Utility functions
 *	Discrete functions used throughout the code
 *
 *	20151110	1.1		Cleanup. Documentation
 *	20151004	1.0		Creation
 */

#ifndef __UTILS_H_
#define __UTILS_H_

#include <stdint.h>


/**Generic success / error code result.
 * This is used widely across the library, in many return values, and in the test suite as well.
 * Only Result_OK means success, any other values mean an error of some sort.
 */
typedef enum
{
	Result_OK = 0,						/* operation completed with success */
	Result_Failed,						/* generic operation failure */
	Result_Failed_NotImplemented,		/* requested feature is not implemented, or supported */
	Result_Failed_InvalidParams,		/* parameters supplied are invalid, unusable, or inconsistent */
	Result_Failed_NotInitialized,		/* could't proceed as there is something not initialized */
	Result_Failed_AlreadyInitialized,	/* initialization was requested for something that is already initialized (thus subject to be in use) */
	Result_Failed_NotEnoughMemory,		/* not enough memory to complete the operation */
	Result_Failed_Cancelled,			/* operation was cancelled */
	Result_Failed_Timeout				/* operation timed out */
}	Result;

enum
{
	Color_Black,
	Color_Red,
	Color_Green,
	Color_Yellow,
	Color_Blue,
	Color_Magenta,
	Color_Cyan,
	Color_White,
	Color_BrightBlack,
	Color_BrightRed,
	Color_BrightGreen,
	Color_BrightYellow,
	Color_BrightBlue,
	Color_BrightMagenta,
	Color_BrightCyan,
	Color_BrightWhite
};


void swap(uint32_t* a, uint32_t* b);

char* makeAnsiColors(char fg, char bg, char* workBuffer);

char* makeAnsiClearScreen();

char* makeAnsiResetAttributes();

Result makeAnsiString(char* dst, char fg, char bg, char* text);



/*	milliseconds to ticks conversion
 *	@param		ms
 *	@return		ticks
 */
uint32_t msToTicks(uint32_t ms);

/*	ticks to milliseconds conversion
 *	@param		ticks
 *	@return		ms
 */
uint32_t ticksToMs(uint32_t ticks);

/*	converts bytes to words
 *	@param		number of bytes
 *	@return		number of words required to contain the bytes
 */

uint32_t bytesToWords(uint32_t bytes);

uint32_t wordsToBytes(uint32_t words);

#endif /* __UTILS_H_ */
