#include "arpalab.h"
#include "wifly.h"
#include "utils.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>


enum { ARPALAB_stringSize = 20 }; 		/* size limit for all module parameters */

char ARPALAB_nid[ARPALAB_stringSize] = { 0 };
char ARPALAB_ip[ARPALAB_stringSize] = { 0 };
char ARPALAB_app[ARPALAB_stringSize] = { 0 };
int ARPALAB_initialized = 0;


Result ARPALAB_init(char* nid, char* ip, char* app)
{
	if(ARPALAB_initialized)
	{
		return Result_Failed_AlreadyInitialized;
	}
	Result result = Result_Failed_InvalidParams;

	if(    (strlen(nid) < ARPALAB_stringSize)
		&& (strlen(ip) < ARPALAB_stringSize)
		&& (strlen(app) < ARPALAB_stringSize) )
	{
		strcpy(ARPALAB_nid, nid);
		strcpy(ARPALAB_ip, ip);
		strcpy(ARPALAB_app, app);
		ARPALAB_initialized = 1;
		result = Result_OK;
	}
	return result;
}

Result ARPALAB_deinit()
{
	if(!ARPALAB_initialized)
	{
		return Result_Failed_NotInitialized;
	}
	ARPALAB_initialized = 0;
	return Result_OK;
}

/*	request example:
 *	http://arpalab.appspot.com/rest/feed
 */
Result ARPALAB_getRandomValue(uint32_t* value)
{
	if(!ARPALAB_initialized)
	{
		return Result_Failed_NotInitialized;
	}
	char* server = "http://arpalab.appspot.com";
	char* url = "/rest/feed";
	enum { bufferSize = 100 };
	char buffer[bufferSize] = { 0 };

	Result result = WIFLY_sendHttpGetRequest(server, url, buffer, bufferSize);
	if(result == Result_OK)
	{
		if(value)
		{
			*value = atoi(buffer);
		}
	}
	return result;
}

/*	request example:
 *	http://arpalab.appspot.com/rest/stats?nid=888888&ip=83.55.225.126&app=test&data=1.345
 */
Result ARPALAB_sendValue(uint32_t value)
{
	if(!ARPALAB_initialized)
	{
		return Result_Failed_NotInitialized;
	}
	char* server = "http://arpalab.appspot.com";
	enum { urlSize = 100 };
	char url[urlSize];
	enum{ bufferSize = 10 };
	char buffer[bufferSize] = { 0 };

	snprintf(url, urlSize, "/rest/stats?nid=%s&ip=%s&app=%s&data=%d", ARPALAB_nid, ARPALAB_ip, ARPALAB_app, value);
	Result result = WIFLY_sendHttpGetRequest(server, url, buffer, bufferSize);
	if(result == Result_OK)
	{	/* ok, response is empty */
	}
	return result;
}

