#include "cmd.h"
#include "log.h"
#include <string.h>


Result CMD_init(CMD_cfg* cfg, char* buffer1, char* buffer2, uint32_t bufferSize, CMD_details* cmdDetails, uint32_t cmdDetailsSize)
{
	if( 	buffer1 == NULL
		||	buffer2 == NULL
		||	bufferSize < 2
		||	cmdDetails == NULL
		||	cmdDetailsSize < 1 )
	{
		return Result_Failed_InvalidParams;
	}
	memset(buffer1, 0, bufferSize);
	memset(buffer2, 0, bufferSize);
	cfg->cmdBuffer = buffer1;
	cfg->workBuffer = buffer2;
	cfg->bufferSize = bufferSize-1;
	CIRCULARBUFFER_init(&cfg->cb, cfg->cmdBuffer, cfg->bufferSize);
	cfg->cmdDetails = cmdDetails;
	cfg->cmdDetailsSize = cmdDetailsSize;
	return Result_OK;
}

Result CMD_processCommand(CMD_cfg* cfg, const char* command_)
{
	uint32_t len, i;
	for(len = 0 ; len < strlen(command_)  ; len++)
	{
//		strcpy (cfg->workBuffer, command_+len);
		const char* w = command_+len;
		for(i=0 ; i < cfg->cmdDetailsSize ; i++)
		{
			CMD_details* d = &(cfg->cmdDetails[i]);
			if(d->isCommand(w, d->command))
			{
				LOG_log("Command '%s' acknowledged. Executing '%s' action.", w, d->description);
				Result cmdResult = d->proc(w, d->params);
				cmdResult = cmdResult;  /* we do nothing with return code */
				return Result_OK; /* ok = a command was processed */
			}
		}
	}
	return Result_Failed; /* no command was processed */
}

Result CMD_processCharacter(CMD_cfg* cfg, char c)
{
	if(Result_OK != CIRCULARBUFFER_pushBack(&cfg->cb, c))
	{
		char tmp;
		CIRCULARBUFFER_popFront(&cfg->cb, &tmp);
		CIRCULARBUFFER_pushBack(&cfg->cb, c);
	}

	char* w = cfg->workBuffer;
	CIRCULARBUFFER_toString(&cfg->cb, w);

	LOG_log("CMD_processCharacter. Accumulated string = %s", w);

	if(CMD_processCommand(cfg, w) == Result_OK)
	{
		CIRCULARBUFFER_clean(&cfg->cb);
	}

	return Result_OK;
}
