#include "gpio.h"
#include "LPC17xx.h"

uint32_t volatile* GPIO_getFioClrReg(GPIO_Port port)
{
	uint32_t volatile* r[] =
	{
		&LPC_GPIO0->FIOCLR,
		&LPC_GPIO1->FIOCLR,
		&LPC_GPIO2->FIOCLR,
		&LPC_GPIO3->FIOCLR,
		&LPC_GPIO4->FIOCLR
	};
	return r[port];
}

uint32_t volatile* GPIO_getFioSetReg(GPIO_Port port)
{
	uint32_t volatile* r[] =
	{
		&LPC_GPIO0->FIOSET,
		&LPC_GPIO1->FIOSET,
		&LPC_GPIO2->FIOSET,
		&LPC_GPIO3->FIOSET,
		&LPC_GPIO4->FIOSET
	};
	return r[port];
}

uint32_t volatile* GPIO_getPinReg(GPIO_Port port)
{
	uint32_t volatile* r[] =
	{
		&LPC_GPIO0->FIOPIN,
		&LPC_GPIO1->FIOPIN,
		&LPC_GPIO2->FIOPIN,
		&LPC_GPIO3->FIOPIN,
		&LPC_GPIO4->FIOPIN
	};
	return r[port];
}

uint32_t volatile* GPIO_getPinModeReg(GPIO_Port port, uint32_t pin)
{
	uint32_t volatile* r[] =
	{
		&LPC_PINCON->PINMODE0,
		&LPC_PINCON->PINMODE1,
		&LPC_PINCON->PINMODE2,
		&LPC_PINCON->PINMODE3,
		&LPC_PINCON->PINMODE4,
		&LPC_PINCON->PINMODE5,
		&LPC_PINCON->PINMODE6,
		&LPC_PINCON->PINMODE7,
		&LPC_PINCON->PINMODE8,
		&LPC_PINCON->PINMODE9
	};
	return r[port*2 + (pin > 15)];
}

uint32_t volatile* GPIO_getDirectionReg(GPIO_Port port)
{
	uint32_t volatile* r[] =
	{
		&LPC_GPIO0->FIODIR,
		&LPC_GPIO1->FIODIR,
		&LPC_GPIO2->FIODIR,
		&LPC_GPIO3->FIODIR,
		&LPC_GPIO4->FIODIR
	};
	return r[port];
}

uint32_t volatile* GPIO_getOpenDrainModeReg(GPIO_Port port)
{
	uint32_t volatile* r[] =
	{
		&LPC_PINCON->PINMODE_OD0,
		&LPC_PINCON->PINMODE_OD1,
		&LPC_PINCON->PINMODE_OD2,
		&LPC_PINCON->PINMODE_OD3,
		&LPC_PINCON->PINMODE_OD4
	};
	return r[port];
}

uint32_t GPIO_getPinValue(GPIO_Port port, uint8_t pin)
{
	uint32_t v = *GPIO_getPinReg(port);
	return (v & (1 << pin)) != 0;
}

void GPIO_setPinValue(GPIO_Port port, uint8_t pin, uint8_t value)
{
	if(value)
	{
		(*GPIO_getFioSetReg(port)) |= (1 << pin);
	}
	else
	{
		(*GPIO_getFioClrReg(port)) |= (1 << pin);
	}
}

Result GPIO_configPinAsOutput(GPIO_Port port, uint8_t pin)
{
	if(pin>31)
	{
		return Result_Failed_InvalidParams;
	}

	/* set pin as output */
	(*GPIO_getDirectionReg(port)) |= (1 << pin);

	return Result_OK;
}

Result GPIO_configPinAsInput(GPIO_Port port, uint8_t pin, GPIO_PinMode pinMode)
{
	if(pin>31)
	{
		return Result_Failed_InvalidParams;
	}

	/* set pin as input */
	(*GPIO_getDirectionReg(port)) &= (~(1 << pin));

	/* set pin mode */
	uint32_t volatile* pinmode = GPIO_getPinModeReg(port, pin);
	uint32_t bit0 = pin * 2;
	uint32_t bit1 = bit0 + 1;

	uint32_t value = *pinmode;
	value &= (~((1 << bit1) | (1 << bit0))); 						/* clear mode bits */
	value |= (((pinMode >> 1) << bit1) | ((pinMode & 1) << bit0));	/* set mode bits */
	*pinmode = value;												/* apply pin mode value */
	return Result_OK;
}

Result GPIO_SetOpenDrainMode(GPIO_Port port, uint8_t pin, uint8_t openDrainMode)
{
	if(pin>31)
	{
		return Result_Failed_InvalidParams;
	}
	/* TODO: validate that is set as input */

	if(openDrainMode)
	{	/* set pin as open drain mode */
		(*GPIO_getOpenDrainModeReg(port)) |= (1 << pin);
	}
	else
	{	/* set pin as 'normal' mode */
		(*GPIO_getOpenDrainModeReg(port)) &= (~(1 << pin));
	}
	return Result_OK;
}

