#include "gpioint.h"
#include "gpio.h"
#include "LPC17xx.h"
#include "utils.h"
#include <stddef.h>
#include <assert.h>


uint32_t GPIOINT_intialized = 0;
GPIOINT_cfg GPIOINT_cfgs[GPIOINT_SlotNum] = {{ 0 }};


uint32_t volatile* GetIntEnF(GPIO_Port port)
{
	uint32_t volatile* r [] =
	{
		&LPC_GPIOINT->IO0IntEnF,
		NULL,
		&LPC_GPIOINT->IO2IntEnF,
		NULL
	};
	return(r[port]);
}

uint32_t volatile* GetIntEnR(GPIO_Port port)
{
	uint32_t volatile* r [] =
	{
		&LPC_GPIOINT->IO0IntEnR,
		NULL,
		&LPC_GPIOINT->IO2IntEnR,
		NULL
	};
	return(r[port]);
}

uint32_t volatile const* GetIntStatR(GPIO_Port port)
{
	uint32_t volatile const* r [] =
	{
		&LPC_GPIOINT->IO0IntStatR,
		NULL,
		&LPC_GPIOINT->IO2IntStatR,
		NULL
	};
	return(r[port]);
}

uint32_t volatile const* GetIntStatF(GPIO_Port port)
{
	uint32_t volatile const* r [] =
	{
		&LPC_GPIOINT->IO0IntStatF,
		NULL,
		&LPC_GPIOINT->IO2IntStatF,
		NULL
	};
	return(r[port]);
}

uint32_t volatile* GetIntClr(GPIO_Port port)
{
	uint32_t volatile* r [] =
	{
		&LPC_GPIOINT->IO0IntClr,
		NULL,
		&LPC_GPIOINT->IO2IntClr,
		NULL
	};
	return(r[port]);
}

uint8_t GPIOINT_isInitialized(void)
{
	return GPIOINT_intialized != 0;
}

Result GPIOINT_init(void)
{
	if(GPIOINT_isInitialized())
	{
		return Result_Failed_AlreadyInitialized;
	}
	GPIOINT_intialized = 1;
	NVIC_EnableIRQ(EINT3_IRQn);
	return Result_OK;
}

Result GPIOINT_deinit(void)
{
	if(!GPIOINT_isInitialized())
	{
		return Result_Failed_NotInitialized;
	}

	int i;
	for(i = 0; i < GPIOINT_SlotNum  ; i++)
	{
		GPIOINT_cfg* cfg = &GPIOINT_cfgs[i];
		if(cfg->handler != NULL)
		{
			GPIOINT_unset(i);
		}
	}

	GPIOINT_intialized = 0;
	NVIC_DisableIRQ(EINT3_IRQn);
	return Result_OK;
}

Result GPIOINT_set(GPIOINT_Slot slot, GPIOINT_cfg* userCfg)
{
	if(!GPIOINT_isInitialized())
	{
		return Result_Failed_NotInitialized;
	}
	if(slot >= GPIOINT_SlotNum)
	{
		return Result_Failed_InvalidParams;
	}
	if(userCfg->handler == NULL)
	{
		return Result_Failed_InvalidParams;
	}
	if(userCfg->pin > 31)
	{
		return Result_Failed_InvalidParams;
	}
	if(userCfg->port != GPIO_Port0 && userCfg->port != GPIO_Port2)
	{
		return Result_Failed_InvalidParams;	/* external interrupt can only be set to P0 or P2 */
	}

	GPIOINT_cfg* cfg = &GPIOINT_cfgs[slot];
	if(cfg->handler != NULL)				/* handler is used to check for already set */
	{
		return Result_Failed_AlreadyInitialized;
	}
/*
	if((*GetIntEnR(cfg->port)) & (1 << cfg->pin))
	{
		return Result_Failed_AlreadyInitialized;
	}
	if((*GetIntEnF(cfg->port)) & (1 << cfg->pin))
	{
		return Result_Failed_AlreadyInitialized;
	}
*/

	cfg->handler  = userCfg->handler;
	cfg->params   = userCfg->params;
	cfg->port     = userCfg->port;
	cfg->pin      = userCfg->pin;
	cfg->edgeKind = userCfg->edgeKind;

	(*GetIntClr(cfg->port)) |= (1 << cfg->pin);

	switch(cfg->edgeKind)
	{
		case GPIOINT_EdgeKind_None:
			(*GetIntEnR(cfg->port)) &= ~(1 << cfg->pin);
			(*GetIntEnF(cfg->port)) &= ~(1 << cfg->pin);
			break;
		case GPIOINT_EdgeKind_Raising:
			(*GetIntEnR(cfg->port)) |=  (1 << cfg->pin);
			(*GetIntEnF(cfg->port)) &= ~(1 << cfg->pin);
			break;
		case GPIOINT_EdgeKind_Falling:
			(*GetIntEnR(cfg->port)) &= ~(1 << cfg->pin);
			(*GetIntEnF(cfg->port)) |=  (1 << cfg->pin);
			break;
		case GPIOINT_EdgeKind_RaisingORFalling:
			(*GetIntEnR(cfg->port)) |=  (1 << cfg->pin);
			(*GetIntEnF(cfg->port)) |=  (1 << cfg->pin);
			break;
	}
	return Result_OK;
}

Result GPIOINT_unset(GPIOINT_Slot slot)
{
	if(!GPIOINT_isInitialized())
	{
		return Result_Failed_NotInitialized;
	}
	if(slot >= GPIOINT_SlotNum)
	{
		return Result_Failed_InvalidParams;
	}

	GPIOINT_cfg* cfg = &GPIOINT_cfgs[slot];
	if(cfg->handler == NULL)		/* handler is used to check for already set */
	{
		return Result_Failed_NotInitialized;
	}

	/* TODO: unset edge registers ? */

	(*GetIntEnR(cfg->port)) &= ~(1 << cfg->pin);
	(*GetIntEnF(cfg->port)) &= ~(1 << cfg->pin);

	cfg->handler = NULL;
	return Result_OK;
}

void EINT3_IRQHandler(void)
{
	uint32_t i;
	for(i = 0; i < GPIOINT_SlotNum  ; i++)
	{
		GPIOINT_cfg* cfg = &GPIOINT_cfgs[i];
		if(cfg->handler != NULL)
		{
			uint32_t egde = 0;
			switch(cfg->edgeKind)
			{
				case GPIOINT_EdgeKind_None:
					break;
				case GPIOINT_EdgeKind_Raising:
					egde  = ((*GetIntStatR(cfg->port)) & (1 << cfg->pin));
					break;
				case GPIOINT_EdgeKind_Falling:
					egde = ((*GetIntStatF(cfg->port)) & (1 << cfg->pin));
					break;
				case GPIOINT_EdgeKind_RaisingORFalling:
					egde  = 	((*GetIntStatR(cfg->port)) & (1 << cfg->pin))
							||	((*GetIntStatF(cfg->port)) & (1 << cfg->pin));
					break;
			}

			if(egde)
			{
				uint8_t pinValue = 0 != GPIO_getPinValue(cfg->port, cfg->pin);
				cfg->handler(cfg->params, pinValue);
				(*GetIntClr(cfg->port)) |= (1 << cfg->pin);
			}
		}
	}
}
