#include "LPC17xx.h"
#include "keypad.h"
#include <string.h>


Result KEYPAD_init(const KEYPAD_cfg* cfg)
{
	/* TODO: check pin values from config? */
	/* TODO validate size vs keyList */
	if(strlen(cfg->keyList) != cfg->keyListSize)
	{
		return Result_Failed_InvalidParams;
	}

	Result result;
	uint32_t p;
	for(p = 0 ; p < cfg->countA ; p++)
	{
		result = GPIO_configPinAsInput(cfg->portA, cfg->startA + p, PinMode_PullDown);
		if(Result_OK != result)
		{
			return result;
		}
	}
	for(p = 0 ; p < cfg->countB ; p++)
	{
		result = GPIO_configPinAsOutput(cfg->portB, cfg->startB + p);
		if(Result_OK != result)
		{
			return result;
		}
	}
	return Result_OK;
}

Result KEYPAD_scan(const KEYPAD_cfg* cfg, char* keys)
{
	uint32_t a, b, a2, b2;

	memset(keys, cfg->noKey, cfg->keyListSize);

	for(b = 0 ; b < cfg->countB ; b++)
	{
		for(b2 = 0 ; b2 < cfg->countB ; b2++)
		{
			GPIO_setPinValue(cfg->portB, cfg->startB + b2, b==b2);
		}

		for(a = 0 ; a < cfg->countA ; a++)
		{
			for(a2 = 0 ; a2 < cfg->countA ; a2++)
			{
				GPIO_SetOpenDrainMode(cfg->portA, cfg->startA + a2, a!=a2);
			}

			if(GPIO_getPinValue(cfg->portA, cfg->startA + a))
			{
				uint32_t i = cfg->computeIndexCallback(a, b, cfg->countA, cfg->countB);
				keys[i] = cfg->keyList[i];
			}
		}
	}

	/* keep all pins in the same state */
	GPIO_setPinValue(cfg->portB, cfg->startB + cfg->countB -1, 0);
	GPIO_SetOpenDrainMode(cfg->portA, cfg->startA + cfg->countA -1, 1);
	return Result_OK;
}

char KEYPAD_getKeypress(const KEYPAD_cfg* cfg, const char* keys)
{
	char r = 0;
	int i;
	for(i=0 ; i<cfg->keyListSize ; i++)
	{
		if(keys[i] != cfg->noKey)
		{
			if(r)
			{
				return 0; /* two characters in buffer: return 0 means "no char acknowledged" */
			}
			else
			{
				r = keys[i];
			}
		}
	}
	return r; /* either 0 or the one valid key */
}
