#include "FreeRTOS.h"
#include "task.h"
#include "uart.h"
#include "log.h"
#include "printf.h"
#include "utils.h"
#include <assert.h>
#include <stdarg.h>
#include <string.h>
#include <stdio.h>


UART_port LOG_port = UART_portNone;


/** Generate a log trace header
 * @precondition the pBuffer is a valid pointer and has enough space
 *
 * @param header			pointer to a buffer that will receive the header
 * @param headerMaxSize     max size to fill in header, in characters.
 *
 */
void LOG_makeTraceHeader(char* header, size_t headerMaxSize)
{
	snprintf
	(
		header,
		headerMaxSize,
		"0x%08X %08u [%-12s] ",	/* 12 characters as it is configMAX_TASK_NAME_LEN */
		xTaskGetCurrentTaskHandle(),
		xTaskGetTickCountFromISR(),
		pcTaskGetTaskName(NULL)
	);
}

Result LOG_init(UART_port port)
{
	if(LOG_port != UART_portNone)
	{
		return Result_Failed_AlreadyInitialized;
	}
	if(!UART_isInitialized(port))
	{
		return Result_Failed;
	}
	if(!PRINTF_isInitialized(port))
	{
		return Result_Failed;
	}

	LOG_port = port;
	return Result_OK;
}

Result LOG_log(const char* format, ...)
{
	if(LOG_port == UART_portNone)
	{
		return Result_Failed_NotInitialized;
	}

	enum
	{
		formatExMaxSize	= 128,
		headerMaxSize	= 64
	};
	char formatEx[formatExMaxSize] = {0};
	char header[headerMaxSize] = { 0 };
	LOG_makeTraceHeader(header, headerMaxSize);

	Result result = Result_Failed_NotEnoughMemory;
	int formatExSizeRequired = strlen(header) + strlen(format) + 2; /* 2 extra characters for CR+LF */
	if (formatExSizeRequired < formatExMaxSize)
	{
		strcpy(formatEx, header);
		strcat(formatEx, format);
		strcat(formatEx, "\r\n");

		va_list argList;				/* variable parameters usage */
		va_start(argList, format);
		result = PRINTF_printVa(LOG_port, formatEx, argList);
		va_end(argList);
	}
	return result;
}

Result LOG_logVa(const char* format, ...)
{
	if(LOG_port == UART_portNone)
	{
		return Result_Failed_NotInitialized;
	}

	enum
	{
		formatExMaxSize	= 128,
		headerMaxSize	= 64
	};
	char formatEx[formatExMaxSize] = {0};
	char header[headerMaxSize] = { 0 };
	LOG_makeTraceHeader(header, headerMaxSize);

	Result result = Result_Failed_NotEnoughMemory;
	int formatExSizeRequired = strlen(header) + strlen(format) + 2; /* 2 extra characters for CR+LF */
	if (formatExSizeRequired < formatExMaxSize)
	{
		strcpy(formatEx, header);
		strcat(formatEx, format);
		strcat(formatEx, "\r\n");

		va_list argList;				/* variable parameters usage */
		va_start(argList, format);
		result = PRINTF_printVa(LOG_port, formatEx, argList);
		va_end(argList);
	}
	return result;
}
