#include "parsing.h"
#include "FreeRTOS.h"
#include <string.h>


Result PARSING_checkSubst(const char *buffer, int start, const char* match)
{
	int bufferLength = strlen(buffer);
	int matchLength = strlen(match);

	if(bufferLength==0 || matchLength==0)
	{
		return Result_Failed_InvalidParams;
	}

	if(start<0)			/* negative start means reverse-indexing */
	{
		start = bufferLength + start;
	}
	if(start < 0 || start > bufferLength-matchLength)
	{
		return Result_Failed_InvalidParams;
	}

	size_t i;
	for (i = 0; i < matchLength ; i++)
	{
		if(buffer[i+start] != match[i])
		{
			return Result_Failed;
		}
	}
	return Result_OK;
}

Result PARSING_splitLines(char* buffer, char** lines, size_t linesSize)
{
	Result result = Result_OK;
	int i;
	int lineCount = 0;
	int dataCount = 0;
	int done = 0;
	char *start = buffer;

	if((lines==NULL) != (linesSize==0))
	{
		return Result_Failed_InvalidParams;
	}

	for(i = 0 ; !done/*buffer[i]*/ != 0 ; i ++)
	{
		switch(buffer[i])
		{
			case 0:
				done = 1;
		/*	do not break */
			case '\r':
			case '\n':
				if(dataCount > 0)
				{
					if(lineCount < linesSize)
					{
						if(lines)
						{
							lines[lineCount++] = start;
						}
					}
					else
					{
						result = Result_Failed_NotEnoughMemory;
					}
				}
				start = &buffer[i+1];
				buffer[i] = 0;
				dataCount = 0;
				break;
			default:
				dataCount++;
				break;
		}
	}

	/* set remaining lines as empty */
	if(lines)
	{
		while(lineCount < linesSize)
		{
			lines[lineCount++] = NULL;
		}
	}
	return result;
}

#if 0 /* deprecated */

char* PARSING_removeEnclosingTags(char* buffer, const char *left, const char* right)
{
	size_t bufferLen = strlen(buffer);
	size_t leftLen = strlen(left);
	size_t rightLen = strlen(right);
	if(strncmp(buffer, left, leftLen) != 0)
	{
		return NULL;
	}
	if(strcmp(buffer + bufferLen - rightLen, right) != 0)
	{
		return NULL;
	}
	buffer[bufferLen - rightLen] = 0;
	return buffer + leftLen;
}

#else

char* PARSING_removeEnclosingTags(char* buffer, const char *left, const char* right)
{
	char* result = NULL;
	if(left && right)
	{
		char* pleft = strstr(buffer, left);
		if(pleft)
		{
			char* pright = NULL;
			if(strlen(right) == 0)
			{
				pright = buffer + strlen(buffer);
			}
			else
			{
				pright = (char*)strstr(pleft + strlen(left), right);

				while(pright)
				{
					char *rightest = (char*)strstr(pright + strlen(right), right);
					if(rightest)
					{
						pright = rightest;
					}
					else
					{
						break;
					}
				}
			}

			if(pleft && pright)
			{
				*pright = 0;		/* modifies buffer */
				result = (pleft + strlen(left));
			}
		}
	}
	return result;
}

#endif
