#include "utils.h"
#include "FreeRTOS.h"
#include <string.h>
#include <stdio.h>


enum { wordSize = 4 };


#define ALLOW_ANSI_ESCAPE_SEQUENCES 1 	/* 0:disallow, 1:allow  (ansi escape sequence commands) */


/* TODO: pass buffer size, use secure printf */
char* makeAnsiColors(char fg, char bg, char* workBuffer)
{
#if ALLOW_ANSI_ESCAPE_SEQUENCES
	sprintf(workBuffer, "\x1b[%d%s;%dm", 30 + (fg % 8), (fg > 7) ? ";1" : "", 40 + (bg % 8));
	return workBuffer;
#else
		return "";
#endif
}

char* makeAnsiClearScreen()
{
#if ALLOW_ANSI_ESCAPE_SEQUENCES
	return "\x1b[2J";
#else
	return "";
#endif
}

char* makeAnsiResetAttributes()
{
#if ALLOW_ANSI_ESCAPE_SEQUENCES
	return "\x1b[0m";
#else
	return "";
#endif
}

Result makeAnsiString(char* dst, char fg, char bg, char* text)
{
	char tmp[30];
	sprintf(dst, "%s%s%s", makeAnsiColors(fg, bg, tmp), text, makeAnsiResetAttributes());
	return Result_OK;  /* TODO: check sizes */
}

void swap(uint32_t* a, uint32_t* b)
{
	uint32_t tmp = *a;
	*a = *b;
	*b = tmp;
}


uint32_t msToTicks(uint32_t ms)
{
	return ms / (double) portTICK_RATE_MS;
}

uint32_t ticksToMs(uint32_t ticks)
{
	return ticks * portTICK_RATE_MS;
}

uint32_t bytesToWords(uint32_t bytes)
{
	return (bytes / wordSize) + (bytes % wordSize != 0);
}

uint32_t wordsToBytes(uint32_t words)
{
	return words * wordSize;
}
