/* TEST module
 * Minimal test/test_suite infrastructure framework
 *
 * 20151107		1.2		Check utilities added
 * 20151101		1.1		Global state. task termination issues fixed
 * 20151025		1.0		Creation
 */

#ifndef __TESTFRAMEWORK_H
#define __TESTFRAMEWORK_H

#include "FreeRTOS.h"
#include "task.h"
#include "utils.h"


typedef enum
{
	TestTaskState_Watiting,
	TestTaskState_Running,
	TestTaskState_Completed
}	TestTaskState;

typedef enum
{
	TestResult_Succeeded,
	TestResult_Failed,
	TestResult_NotStarted,
	TestResult_Started,
}	TestResult;

typedef struct
{
/*	parameters for xTaskCreate() */
	void (*taskProc)(void*);
	const char const* taskName;
	int stackDepth;
	void* params;
	UBaseType_t priority;
/*	test details - members below are initialized @ Run(Test*) */
	xTaskHandle handle;
	int totalChecks;
	int failedChecks;
	volatile TestTaskState state;
}	Test;

typedef struct
{
	const char const * name;
	Test* tests;
	size_t testsSize;
	int stackDepth;
	UBaseType_t priority;
	int logPort;
	int logPortBauds;
/*	test suite details - members below are initialiazed @ RunSuite(TestSuite*) */
	int totalChecks;
	int failedChecks;
	int totalTests;
	int failedTests;
}	TestSuite;


/**Facilities for in-test usage
 * Invoke CheckTestInt() / CheckResult() with provided result.
 * Handle file/line transparently, to referentiate individual checks
 *
 * @param	result	see CheckTestInt() / CheckResult() 's result param
 * */
#define TEST_CheckResult(result)		TEST_CheckResultFL(result, __FILE__, __LINE__)
#define TEST_CheckInt(result)			TEST_CheckIntFL(result, __FILE__, __LINE__)

/**Track a test check result
 *
 * @param	result	result for a test check. Any result other than Result_OK is acknowledged as failed check
 * @param	file	source file where the check is
 * @param	line	line within the source where the check
 * */
void TEST_CheckResultFL(Result result, char* file, int line);

/**Track a test check result
 *
 * @param	result	result for a test check. Any result other than 0 is acknowledged as passed check
 * @param	file	source file where the check is
 * @param	line	line within the source where the check
 * */
void TEST_CheckIntFL(int result, char* file, int line);

/**Test suite runner
 * Creates a task where tests are run sequentially
 * Initializes logger
 * Keeps track of test/test_suite results
 *
 * @param	TestSuite	details for the test suite
 * */
void TEST_runSuite(TestSuite* testSuite);


#endif /* __TESTFRAMEWORK_H */
