#include "FreeRTOS.h"
#include "task.h"
#include <assert.h>
#include <stdio.h>
#include <string.h>
#include "utils.h"
#include "cmd.h"
#include "keypad.h"
#include "printf.h"
#include "log.h"
#include "test_framework.h"
#include "test_helpers.h"
#include "test_config.h"

/* Support functions for manualTest_cmd() */

static uint32_t computeIndexCallback(uint32_t a, uint32_t b, uint32_t countA, uint32_t countB)
{
	return (a * countB) + b;
}

static uint32_t IsCommand(const char* userInput, const char* cmd)
{
	return 0 == strcmp(userInput, cmd);
}

static Result f1(const char* command, void* params)
{
	uint32_t* quit = (uint32_t*)params;
	*quit = 1;
	LOG_log("User entered command f1234 - quitting test");
	return Result_OK;
}

static Result f2(const char* command, void* params)
{
	LOG_log("User entered command f2345");
	return Result_OK;
}

static Result f3(const char* command, void* params)
{
	LOG_log("User entered command f333");
	return Result_OK;
}

/* Manual test for the CMD module. See in-test instructions for details */
void manualTest_cmd(void* pvParameters)
{
	assert(pvParameters == NULL);
	char* instructions [] =
	{
		"Manual test for the CMD module.",
		"To finish this test, enter the '1234' command.",
		"After entering one of the commands below, check that it is properly acknowledged as such.",
		"A logging statement should be emited whenever a command is entered."
		"Available commands are:",
		"    1234     - this command quits the test",
		"    2345",
		"    333",
		NULL
	};
	showInstructions(instructions);

	uint32_t quit = 0;
	enum { bufferSize = 10, commandDetailsSize = 3  };
	char buffers[2][bufferSize] = {{ 0 }};
	CMD_details commandDetails[commandDetailsSize] =
	{
		{ IsCommand, "1234", f1, &quit, "f1()" },
		{ IsCommand, "2345", f2, NULL,  "f2()" },
		{ IsCommand, "333",  f3, NULL,  "f3()" }
	};
	CMD_cfg cmdCfg = { 0 };
	TEST_CheckResult(CMD_init(&cmdCfg, buffers[0], buffers[1], bufferSize, commandDetails, commandDetailsSize));

	const KEYPAD_cfg keypadCfg =
	{
		testConfig_PortB, testConfig_PinStartKeypadA, testConfig_PinSizeKeypadA,
		testConfig_PortB, testConfig_PinStartKeypadB, testConfig_PinSizeKeypadB,
		3*4, "123456789*0#", ' ', computeIndexCallback
	};
	TEST_CheckResult(KEYPAD_init(&keypadCfg));

	while(!quit)
	{
		char keys[3*4] = { 0 };
		KEYPAD_scan(&keypadCfg, keys);
		char key = KEYPAD_getKeypress(&keypadCfg, keys);
		if(key)
		{
			PRINTF_print(testConfig_loggerPort, "%c", key);
			CMD_processCharacter(&cmdCfg, key);

			while(key) /* eat all keypresses until no key is pressed */
			{
				KEYPAD_scan(&keypadCfg, keys);
				key = KEYPAD_getKeypress(&keypadCfg, keys);
			}
		}
		vTaskDelay(msToTicks(100));
	}
}
