#include "test_helpers.h"
#include "test_framework.h"
#include "test_config.h"
#include "log.h"
#include "gpio.h"
#include "utils.h"
#include "keypad.h"
#include "printf.h"
#include <assert.h>
#include <string.h>

/**GPIO test.
 * Set pins to certain values, then reads other pins which are connected, values should match with the ones set.
 */
void unitTest_selfGpio(void* pvParameters)
{
	assert(pvParameters == NULL);
	GPIO_Port p = testConfig_PortB;
	uint32_t i = testConfig_PinIA;
	uint32_t o = testConfig_PinOA;

	GPIO_PinMode mode;
	uint32_t swapCount;
	for(mode = PinMode_PullUp ; mode <= PinMode_PullDown ; mode++)
	for(swapCount = 0 ; swapCount < 2 ; swapCount++)
	{
		LOG_log("mode=%d swap-count=%d %d->%d", mode, swapCount, o, i);
		TEST_CheckResult(GPIO_configPinAsInput(p, i, mode));
		TEST_CheckResult(GPIO_configPinAsOutput(p, o));

		GPIO_setPinValue(p, o, 1);
		TEST_CheckInt(1 == GPIO_getPinValue(p, i));

		GPIO_setPinValue(p, o, 0);
		TEST_CheckInt(0 == GPIO_getPinValue(p, i));

		GPIO_setPinValue(p, o, 1);
		TEST_CheckInt(1 == GPIO_getPinValue(p, i));

		GPIO_setPinValue(p, o, 0);
		TEST_CheckInt(0 == GPIO_getPinValue(p, i));

		swap(&i, &o);
	}
}

/* Manual test for the GPIO module. See in-test instructions for details. */
void manualTest_gpio(void* pvParameters)
{
	assert(pvParameters == NULL);
	char* instructions [] =
	{
		"Manual test for GPIO module.",
		"To finish this test, press '*' in the keypad.",
		"This test polls MC sensor (P2.13) every 200ms and displays the read value (0/1).",
		"Also, red/green LEDS reflect the sensor's read value.",
		NULL
	};
	showInstructions(instructions);

#if 1 /* poll MC sensor */
	LOG_log("checking MC sensor...");
	GPIO_Port port = testConfig_PortB;
	uint32_t pin = testConfig_PinID;
#else /* (alternativelly) poll PIR sensor */
	LOG_log("checking PIR sensor...");
	GPIO_Port port = GPIO_Port0;
	uint32_t pin = 26;
#endif

	GPIO_Port portLEDs = testConfig_PortA;
	uint32_t oRed = testConfig_PinOF;
	uint32_t oGreen = testConfig_PinOE;

	TEST_CheckResult(GPIO_configPinAsInput(port, pin, PinMode_PullUp));
	TEST_CheckResult(GPIO_configPinAsOutput(portLEDs, oRed));
	TEST_CheckResult(GPIO_configPinAsOutput(portLEDs, oGreen));

	TEST_CheckResult(KEYPAD_init(&testKeypadCfg));
	char keys[3*4] = { 0 };
	do KEYPAD_scan(&testKeypadCfg, keys); while(keys[9] == '*');

	while(keys[9] != '*')
	{
		KEYPAD_scan(&testKeypadCfg, keys);

		uint32_t value = GPIO_getPinValue(port, pin);

		GPIO_setPinValue(portLEDs, oRed, value == 0);
		GPIO_setPinValue(portLEDs, oGreen, value != 0);

		PRINTF_print(testConfig_loggerPort, "%d", value);
		vTaskDelay(msToTicks(200));
	}
	PRINTF_print(testConfig_loggerPort, "\r\n");
}
