#include <assert.h>
#include <stdio.h>
#include "test_framework.h"
#include "test_helpers.h"
#include "test_config.h"
#include "ledctrl.h"
#include "utils.h"
#include "log.h"
#include "FreeRTOS.h"
#include "task.h"

/* Support functions for manualTest_ledctrl() */

static void configureLeds(uint32_t value, char* patternRed, char* patternGreen)
{
	uint32_t redOn = 10 * value / 9;
	uint32_t redOff = 10 - redOn;
	uint32_t greenT = value * 5;
	LOG_log("red: on/off=%d/%d%%      green: T=%dms", redOn, redOff, greenT);

	LEDCTRL_config(LEDCTRL_LedRed, patternRed, redOn, redOff);
	LEDCTRL_config(LEDCTRL_LedGreen, patternGreen, greenT/2, greenT/2);
}

/* Manual test for the LEDCTRL module. See in-test instructions for details */
void manualTest_ledctrl(void* pvParameters)
{
	assert(pvParameters == NULL);
	char* instructions [] =
	{
		"Manual test for LEDCTRL module.",
		"To finish this test, press '*' in the keypad.",
		"Keypad controls the green (P0.21) and red (P0.22) LEDs, using keys '0'-'9':",
		"  - For the green LED, control the blinking period.",
		"    Setting a period of 0 renders the led non-updatable",
		"  - For the red LED, controls its ON/OFF ratio.",
		"    Note that it always blinks at a fixed frequency (1 sec)",
		"    Also, a  '0' sets led 100% off, '9' sets it 100% on.",
		"Plus, '#' inserts a specific pattern (fast intermitency which lasts 1 second) for each LED.",
		NULL
	};
	showInstructions(instructions);

	char keys[3*4] = { 0 };
	KEYPAD_init(&testKeypadCfg);
	do KEYPAD_scan(&testKeypadCfg, keys); while(keys[9] == '*');
	uint32_t cfg = 5;

	TEST_CheckResult(LEDCTRL_init(100, testConfig_PortA, testConfig_PinOF, testConfig_PortA, testConfig_PinOE));
	configureLeds(cfg, NULL, NULL);

	uint8_t quit = 0;
	while(!quit)
	{
		/*Result result =  */ KEYPAD_scan(&testKeypadCfg, keys);
		char key = KEYPAD_getKeypress(&testKeypadCfg, keys);
		switch(key)
		{
			case '0': case '1': case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9':
				cfg = key - '0';
				configureLeds(cfg, NULL, NULL);
				break;
			case '#':
				configureLeds(cfg, "010101010", "100110111");
				break;
			case '*':
				quit = 1;
				break;
		}
		vTaskDelay(msToTicks(100));
	}
	TEST_CheckResult(LEDCTRL_deinit());
}

/**LEDCTRL test on the LEDCTRL_init/LEDCTRL_deinit
 * This is a minimal test. There are many more cases, that even include calling some API functions for the LEDCTRL module.
 */
void unitTest_ledctrlInit(void* pvParameters)
{
	assert(pvParameters == NULL);
	TEST_CheckInt(LEDCTRL_init(50, testConfig_PortA, testConfig_PinOE, testConfig_PortA, testConfig_PinOF) == Result_OK);
	TEST_CheckInt(LEDCTRL_init(50, testConfig_PortA, testConfig_PinOE, testConfig_PortA, testConfig_PinOF) == Result_Failed_AlreadyInitialized);

	TEST_CheckInt(LEDCTRL_deinit() == Result_OK);
	TEST_CheckInt(LEDCTRL_deinit() == Result_Failed_NotInitialized);
}
