#include "utils.h"
#include "parsing.h"
#include "test_framework.h"
#include "test_config.h"
#include <assert.h>
#include <string.h>


/* Test for PARSING_splitLines() */
void unitTest_splitLines(void* pvParameters)
{
	assert(pvParameters == NULL);
	char* l[4]  = { 0 }; /* lines */
	char  b[12] = { 0 }; /* buffer */

	strcpy(b,"");
	memset(l, -1, sizeof(l));
	TEST_CheckInt(PARSING_splitLines(b, l, 4) == Result_OK);
	TEST_CheckInt(l[0] == NULL);

	strcpy(b,"\r\n");
	memset(l, -1, sizeof(l));
	TEST_CheckInt(PARSING_splitLines(b, l, 4) == Result_OK);
	TEST_CheckInt(l[0] == NULL);

	strcpy(b,"\r");
	memset(l, -1, sizeof(l));
	TEST_CheckInt(PARSING_splitLines(b, l, 4) == Result_OK);
	TEST_CheckInt(l[0] == NULL);

	strcpy(b,"\n");
	memset(l, -1, sizeof(l));
	TEST_CheckInt(PARSING_splitLines(b, l, 4) == Result_OK);
	TEST_CheckInt(l[0] == NULL);

	strcpy(b,"123");
	memset(l, -1, sizeof(l));
	TEST_CheckInt(PARSING_splitLines(b, l, 4) == Result_OK);
	TEST_CheckInt(strcmp(l[0], "123") == 0);

	strcpy(b,"\r\n123");
	memset(l, -1, sizeof(l));
	TEST_CheckInt(PARSING_splitLines(b, l, 4) == Result_OK);
	TEST_CheckInt(strcmp(l[0], "123") == 0);
	TEST_CheckInt(l[1] == NULL);

	strcpy(b,"\r\n\r\n123");
	memset(l, -1, sizeof(l));
	TEST_CheckInt(PARSING_splitLines(b, l, 4) == Result_OK);
	TEST_CheckInt(strcmp(l[0], "123") == 0);
	TEST_CheckInt(l[1] == NULL);

	strcpy(b,"123\r\n");
	memset(l, -1, sizeof(l));
	TEST_CheckInt(PARSING_splitLines(b, l, 4) == Result_OK);
	TEST_CheckInt(strcmp(l[0], "123") == 0);
	TEST_CheckInt(l[1] == NULL);

	strcpy(b,"123\r");
	memset(l, -1, sizeof(l));
	TEST_CheckInt(PARSING_splitLines(b, l, 4) == Result_OK);
	TEST_CheckInt(strcmp(l[0], "123") == 0);
	TEST_CheckInt(l[1] == NULL);

	strcpy(b,"123\n");
	memset(l, -1, sizeof(l));
	TEST_CheckInt(PARSING_splitLines(b, l, 4) == Result_OK);
	TEST_CheckInt(strcmp(l[0], "123") == 0);
	TEST_CheckInt(l[1] == NULL);

	strcpy(b,"123\r\n456");
	memset(l, -1, sizeof(l));
	TEST_CheckInt(PARSING_splitLines(b, l, 4) == Result_OK);
	TEST_CheckInt(strcmp(l[0], "123") == 0);
	TEST_CheckInt(strcmp(l[1], "456") == 0);
	TEST_CheckInt(l[2] == NULL);

	strcpy(b,"123\r456");
	memset(l, -1, sizeof(l));
	TEST_CheckInt(PARSING_splitLines(b, l, 4) == Result_OK);
	TEST_CheckInt(strcmp(l[0], "123") == 0);
	TEST_CheckInt(strcmp(l[1], "456") == 0);
	TEST_CheckInt(l[2] == NULL);

	strcpy(b,"123\n456");
	memset(l, -1, sizeof(l));
	TEST_CheckInt(PARSING_splitLines(b, l, 4) == Result_OK);
	TEST_CheckInt(strcmp(l[0], "123") == 0);
	TEST_CheckInt(strcmp(l[1], "456") == 0);
	TEST_CheckInt(l[2] == NULL);

	strcpy(b,"123\r\n456");
	memset(l, -1, sizeof(l));
	TEST_CheckInt(PARSING_splitLines(b, l, 4) == Result_OK);
	TEST_CheckInt(strcmp(l[0], "123") == 0);
	TEST_CheckInt(strcmp(l[1], "456") == 0);
	TEST_CheckInt(l[2] == NULL);

	strcpy(b,"123\r\n456\r\n789");
	memset(l, -1, sizeof(l));
	TEST_CheckInt(PARSING_splitLines(b, l, 2) == Result_Failed_NotEnoughMemory);
	TEST_CheckInt(strcmp(l[0], "123") == 0);
	TEST_CheckInt(strcmp(l[1], "456") == 0);

	strcpy(b,"123\r456\n789\r");
	TEST_CheckInt(PARSING_splitLines(b, NULL, 0) == Result_Failed_NotEnoughMemory);
	TEST_CheckInt(b[3] == 0);
	TEST_CheckInt(b[7] == 0);
	TEST_CheckInt(b[11] == 0);

	memset(l, -1, sizeof(l));
	strcpy(b,"123\r456\n789\r");
	TEST_CheckInt(PARSING_splitLines(b, l, 0) == Result_Failed_InvalidParams);
	TEST_CheckInt(b[3] == '\r');
	TEST_CheckInt(b[7] == '\n');	 /* expect no changes*/
	TEST_CheckInt(b[11] == '\r');

	strcpy(b,"123\r456\n789\r");
	TEST_CheckInt(PARSING_splitLines(b, NULL, 1) == Result_Failed_InvalidParams);
	TEST_CheckInt(b[3] == '\r');
	TEST_CheckInt(b[7] == '\n');	/* expect no changes*/
	TEST_CheckInt(b[11] == '\r');
}

/* Test for PARSING_checkSubst() */
void unitTest_checkSubst(void* pvParameters)
{
	assert(pvParameters == NULL);
	char* s = "abcde";

	TEST_CheckInt(PARSING_checkSubst(s,  0,	"a")	== Result_OK);
	TEST_CheckInt(PARSING_checkSubst(s,  0,	"abc")	== Result_OK);
	TEST_CheckInt(PARSING_checkSubst(s,  0,	"a1")	== Result_Failed);
	TEST_CheckInt(PARSING_checkSubst(s,  0,	"1")	== Result_Failed);
	TEST_CheckInt(PARSING_checkSubst(s,  2,	"")		== Result_Failed_InvalidParams);
	TEST_CheckInt(PARSING_checkSubst(s, -2,	"")		== Result_Failed_InvalidParams);
	TEST_CheckInt(PARSING_checkSubst("", 1,	"1")	== Result_Failed_InvalidParams);
	TEST_CheckInt(PARSING_checkSubst(s,  1,	"")		== Result_Failed_InvalidParams);
	TEST_CheckInt(PARSING_checkSubst(s, -2,	"de")	== Result_OK);
	TEST_CheckInt(PARSING_checkSubst(s,  5,	"e")	== Result_Failed_InvalidParams);
	TEST_CheckInt(PARSING_checkSubst(s,  4,	"e")	== Result_OK);
	TEST_CheckInt(PARSING_checkSubst(s, -4,	"bc")	== Result_OK);
	TEST_CheckInt(PARSING_checkSubst(s, -5,	"a")	== Result_OK);
	TEST_CheckInt(PARSING_checkSubst(s, -6,	"1")	== Result_Failed_InvalidParams);
	TEST_CheckInt(PARSING_checkSubst(s,  0,	s)		== Result_OK);
	TEST_CheckInt(PARSING_checkSubst(s, -5,	s)		== Result_OK);
}

/* Test for PARSING_removeEnclosingTags() */
void unitTest_removeEnclosingTags(void* pvParameters)
{
	assert(pvParameters == NULL);
	char s[30] = { 0 };
	char *p = NULL;

	strcpy(s, "(abc)");
	p = PARSING_removeEnclosingTags(s, "(", ")");
	TEST_CheckInt(strcmp(p, "abc") == 0);

	strcpy(s, "-abc)");
	p = PARSING_removeEnclosingTags(s, "(", ")");
	TEST_CheckInt(p == NULL);

	strcpy(s, "(abc-");
	p = PARSING_removeEnclosingTags(s, "(", ")");
	TEST_CheckInt(p == NULL);

	strcpy(s, "<(abc)>");
	p = PARSING_removeEnclosingTags(s, "<(", ")>");
	TEST_CheckInt(strcmp(p, "abc") == 0);

	strcpy(s, "-abc)>");
	p = PARSING_removeEnclosingTags(s, "<(", ")>");
	TEST_CheckInt(p == NULL);

	strcpy(s, "<(abc-");
	p = PARSING_removeEnclosingTags(s, "<(", ")>");
	TEST_CheckInt(p == NULL);

	strcpy(s, "abc");
	p = PARSING_removeEnclosingTags(s, "", "");
	TEST_CheckInt(strcmp(p, "abc") == 0);

	strcpy(s, "()");
	p = PARSING_removeEnclosingTags(s, "(", ")");
	TEST_CheckInt(strcmp(p, "") == 0);

	strcpy(s, "");
	p = PARSING_removeEnclosingTags(s, "(", ")");
	TEST_CheckInt(p == NULL);

	strcpy(s, "123(abc)456");
	p = PARSING_removeEnclosingTags(s, "(", ")");
	TEST_CheckInt(strcmp(p, "abc") == 0);

	strcpy(s, "123abc<(def)>ghi456");
	p = PARSING_removeEnclosingTags(s, "<(", ")>");
	TEST_CheckInt(p != NULL);
	TEST_CheckInt(strcmp(p, "def") == 0);

	strcpy(s, "123(abc(def)ghi)456");
	p = PARSING_removeEnclosingTags(s, "(", ")");
	TEST_CheckInt(p != NULL);
	TEST_CheckInt(strcmp(p, "abc(def)ghi") == 0);

	strcpy(s, "123(abc((def))ghi)456");
	p = PARSING_removeEnclosingTags(s, "((", "))");
	TEST_CheckInt(p != NULL);
	TEST_CheckInt(strcmp(p, "def") == 0);

	strcpy(s, "((((def))))");
	p = PARSING_removeEnclosingTags(s, "((", "))");
	TEST_CheckInt(p != NULL);
	TEST_CheckInt(strcmp(p, "((def))") == 0);
}
