#include "test_helpers.h"
#include "test_framework.h"
#include "test_config.h"
#include "log.h"
#include "gpio.h"
#include "utils.h"
#include "keypad.h"
#include "sensors.h"
#include "printf.h"
#include <assert.h>
#include <string.h>

/* Support functions for manualTest_sensors() */

static void logSensorState(FSM_SensorState* mc, FSM_SensorState* pir)
{
	static uint8_t oldMcDetecting = - 1;
	static uint8_t oldPirDetecting = - 1;

	if((mc->detecting != oldMcDetecting) || (pir->detecting != oldPirDetecting))
	{
		LOG_log  /* logging format replicated from FSM_logSensorStates() */
		(
			"   sensors:   MC( d:%d e:%d )   PIR( d:%d e:%d )",
			mc->detecting, mc->enabled,
			pir->detecting, pir->enabled
		);
		oldMcDetecting = mc->detecting;
		oldPirDetecting = pir->detecting;
	}
}

static void intrHandler(void* params, uint8_t pinValue)
{
	if(params)
	{
		FSM_SensorState* sensorState = (FSM_SensorState*) params;
		sensorState->detecting = pinValue;
	}
}

/**Manual test for the SENSORS module. See in-test instructions for details
 * Note that instead of using SENSORS_init(), this test uses SENSORS_configSensor() directly.
 */
void manualTest_sensors(void* pvParameters)
{
	assert(pvParameters == NULL);
	char* instructions [] =
	{
		"Manual test for SENSORS module",
		"To finish this test, press '*' in the keypad.",
		"Use this test to validate that the CM and PIR sensors detect as it is expected.",
		NULL
	};
	showInstructions(instructions);

	FSM_SensorState mc = {{ 0 }};
	FSM_SensorState pir = {{ 0 }};

	TEST_CheckResult(GPIOINT_init());
	TEST_CheckResult(SENSORS_configSensor(GPIOINT_Slot0, GPIO_Port2, 13, intrHandler, &mc, &mc, 0, "MC "));
	TEST_CheckResult(SENSORS_configSensor(GPIOINT_Slot1, GPIO_Port0, 26, intrHandler, &pir, &pir, 0, "PIR"));

	logSensorState(&mc, &pir);

	TEST_CheckResult(KEYPAD_init(&testKeypadCfg));
	char keys[3*4] = { 0 };
	do KEYPAD_scan(&testKeypadCfg, keys); while(keys[9] == '*');

	while(keys[9] != '*')
	{
		KEYPAD_scan(&testKeypadCfg, keys);
		logSensorState(&mc, &pir);
		vTaskDelay(msToTicks(200));
	}

	TEST_CheckResult(GPIOINT_deinit());
}
