#include "utils.h"
#include <assert.h>
#include "FreeRTOS.h"
#include "task.h"
#include "test_framework.h"
#include "test_helpers.h"
#include "test_config.h"
#include "log.h"
#include "led.h"
#include <string.h>


/**Checks that FreeRTOS library is being compiled with proper tick count support
 * Due to a misconfiguration issue, xTaskGetTickCount() wasn't properly working.
 * This test helped to find that.
 */
void noTest_timings(void* pvParameters)
{
	assert(pvParameters == NULL);

	uint32_t sleepValues [] =  { 10, 100, 1000, 0 }; /* 0 marks end of list */
	uint32_t i, j, j_count = 100*1000;

	uint32_t t1 = xTaskGetTickCount();
	for (j=0 ; j<j_count ; j++) ;
	uint32_t t2 = xTaskGetTickCount();
	uint32_t loopExpected = t2 - t1;

	for(i=0 ; sleepValues[i] ; i++)
	{
		t1 = xTaskGetTickCount();
		vTaskDelay(msToTicks(sleepValues[i]));
		for (j=0 ; j<j_count ; j++) ;
		t2 = xTaskGetTickCount();

		uint32_t totalEllapsed = t2 - t1;
		uint32_t totalExpected = sleepValues[i] + loopExpected;

		double tolerance = 0.1;	/* =10% sa it's 1-based */
		uint32_t maxTotalExpected = totalExpected * (1+tolerance);
		uint32_t minTotalExpected = totalExpected * (1-tolerance);
		TEST_CheckInt((totalEllapsed >= minTotalExpected) && (totalEllapsed <= maxTotalExpected));
	}
}

/**Paints a demonstration palette via log
 * Uses ANSI escape codes. Some terminals are able to display them.
 * Helps a great deal in visibility.
 */
void noTest_palette(void* pvParameters)
{
	assert(pvParameters == NULL);
	char s[200] = "", tmp[30];
	char fg, bg;

	for(bg = 0 ; bg < 16 ; bg++)
	{
		strcpy(s, makeAnsiResetAttributes());
		for(fg = 0 ; fg < 16 ; fg++)
		{
			strcat(s, makeAnsiColors(fg, bg, tmp));
			strcat(s, "X");
		}
		strcat(s, makeAnsiResetAttributes());
		LOG_log("%s", s);
	}
}

/**Sample LED module test.
 * Makes in-board P0.22 LED blink 5 times, in 1 second.
 * Use this as demonstration of test. Although this is not a unit test, since visual inspection of the LED is required.
 */
void noTest_toggleLed(void* pvParameters)
{
	assert(pvParameters == NULL);

	LED_init();
	int i = 0;
	while (i<5)
	{
		LED_toggle();
		LOG_log("led BLINK");
		vTaskDelay(msToTicks(200));
		i++;
	}
	TEST_CheckInt(i==5); /* stupid check. this not unit testing! */
}
