#include "utils.h"
#include "wifly.h"
#include "uart.h"
#include "printf.h"
#include "parsing.h"
#include "test_framework.h"
#include "test_config.h"
#include <assert.h>
#include <string.h>


/* TODO: test uart init-deinit , printf init-deinit */


/* test WIFLY module with scan command issues */
void unitTest_crashScan(void* pvParameters)
{
// TODO: explore this!!!!

	Result WIFLY_receive(char* s, size_t size, uint32_t timeout);
	Result WIFLY_clearBuffer(uint32_t timeout);
	Result WIFLY_send(const char* s);

	assert(pvParameters == NULL);
	//	char buffer[10] = {0};

	TEST_CheckResult(PRINTF_init(testConfig_wiflyPort));			/* TODO: group/nest inits! */
	TEST_CheckResult(UART_init(testConfig_wiflyPort, 9600, UART_modeWithoutInterrupts));
	TEST_CheckResult(WIFLY_init(testConfig_wiflyPort, 0));

	char buffer2[1400];

	WIFLY_clearBuffer(500);
	vTaskDelay(msToTicks(500));
	Result r = WIFLY_send("$$$");
	WIFLY_clearBuffer(500);

	r = WIFLY_send("\rscan 100\r");
	//r = WIFLY_send("get dns\r");
	//r = WIFLY_send("\rget everything\r");
	//r = WIFLY_send("\rget ip\r");
	r = WIFLY_receive(buffer2, sizeof(buffer2), 5000);
	r=0;
	r++;
}

/**WIFLY module test that executes many commands.
 * Reason for having 1 single big test instead of 1 test x command, is that all commands require a long init/connect startup time.
 */
void unitTest_manyCommands(void* pvParameters)
{
	assert(pvParameters == NULL);
	char buffer[10] = {0};

	TEST_CheckResult(PRINTF_init(testConfig_wiflyPort));			/* TODO: group/nest */
	TEST_CheckResult(UART_init(testConfig_wiflyPort, 9600, UART_modeWithInterrupts));
	TEST_CheckResult(WIFLY_init(testConfig_wiflyPort, 1));

	TEST_CheckResult(WIFLY_join("Laurita", "38108497G24354033T"));

	TEST_CheckResult(WIFLY_getIp(buffer)); // no size param

	TEST_CheckResult(WIFLY_deinit(testConfig_wiflyPort));			/* TODO: group/nest */
	TEST_CheckResult(UART_deinit(testConfig_wiflyPort));
	TEST_CheckResult(PRINTF_deinit(testConfig_wiflyPort));
}

/* WIFLY module initialization */
void unitTest_initWifly(void* pvParameters)
{
	assert(pvParameters == NULL);

	vTaskDelay(msToTicks(1000));			/* TODO ! cleanup UARTs? - remove this, if this is handled @ test init */

	TEST_CheckResult(PRINTF_init(testConfig_wiflyPort));
	TEST_CheckResult(UART_init(testConfig_wiflyPort, 9600, UART_modeWithInterrupts));  /* uart must be initialized at 9600, later on we can change baud rate */
	TEST_CheckResult(WIFLY_init(testConfig_wiflyPort, 1));

	TEST_CheckInt(WIFLY_init(testConfig_wiflyPort, 1) == Result_Failed_AlreadyInitialized);

	TEST_CheckResult(WIFLY_deinit(testConfig_wiflyPort));
	TEST_CheckResult(WIFLY_init(testConfig_wiflyPort, 1));			/* TODO: evaluate reboot params in init */

	/* check that WiFly is still responsive */
	char buffer[10] = {0};
	TEST_CheckResult(WIFLY_genericCommand("lites\r", buffer, 10, 1000));
	TEST_CheckInt(PARSING_checkSubst(buffer, 0, "<") == Result_OK && PARSING_checkSubst(buffer, -4, "> ") == Result_OK);
}
